# OpenLaserTag device
# OLT protocol.
# Tomas Krejci [Njord]

import machine, time
from sys import platform
ESP32 = platform == 'esp32'
RP2 = platform == 'rp2'
PYBOARD = platform == 'pyboard'
if ESP32 or RP2:
    from machine import Pin
else:
    from pyb import Pin, LED
import uasyncio as asyncio

#### OLT IR RX imports ###
import ustruct
from sys import platform
import time
import gc
from machine import Pin, freq
from olt_lib.ir_rx.print_error import print_error  # Optional print of error codes
from olt_lib.ir_rx.olt import LT_24 as LT_24_RX # Import all implemented classes

# Define IR RX pin according to platform
if platform == "pyboard":
    ir_rx_pin = Pin("X3", Pin.IN)
elif platform == "esp8266":
    freq(160000000)
    ir_rx_pin = Pin(13, Pin.IN)
elif platform == "esp32" or platform == "esp32_LoBo":
    ir_rx_pin = Pin(23, Pin.IN)
elif platform == "rp2":
    ir_rx_pin = Pin(16, Pin.IN)

#### OLT IR TX imports ###
from sys import platform
ESP32 = platform == 'esp32'
RP2 = platform == 'rp2'
PYBOARD = platform == 'pyboard'
if ESP32 or RP2:
    from machine import Pin
else:
    from pyb import Pin, LED
import uasyncio as asyncio
from primitives.switch import Switch
from primitives.delay_ms import Delay_ms
# Import all implemented classes
from olt_lib.ir_tx.olt import LT_24 as LT_24_TX

# Define IR TX pin according to platform
if ESP32:  # Pins for IR LED gate
    ir_tx_pin = Pin(23, Pin.OUT, value = 0)
elif RP2:
    ir_tx_pin = Pin(17, Pin.OUT, value = 0)
else:
    ir_tx_pin = Pin('X1')

#### Other imports ###
from primitives.delay_ms import Delay_ms
import primitives.queue as queue
from primitives import Switch, Pushbutton
from rgb import RGB

#### OLT Commands ####
LAST_BYTE = (0xE8)
COMMANDS = [['NewGame', 0x83, 0x05],
            ['AdminKill', 0x83, 0x00]]

#### Logging ####
import logging
global logger
init_ticks = time.ticks_ms()
# change logging level here, logging.ERROR is default, logging.DEBUG for more info, and reset device
log_level = logging.DEBUG
#log_level = logging.ERROR
logging.basicConfig(level=log_level, format='%(name)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)
#PYTHONASYNCIODEBUG=1

### alive check ###
async def alive_check(): 
    # loop for checked is alive
    logger.debug("alive check start")
    onboar_led = machine.Pin("LED", machine.Pin.OUT) # Pi Pico onboard LED
    
    
    while True: 
        logger.debug(f"running ms: {time.ticks_ms() - init_ticks}")      
        #print(f"main while running ms: {time.ticks_ms() - init_ticks}")
        onboar_led.on()
        await asyncio.sleep_ms(100)
        onboar_led.off()
        await asyncio.sleep_ms(900)    

### OLT Start game command functions ###
async def start_game_comand(btn, rgb, ir_tx):
    logger.debug("Start game command on button press")
    await rgb.set_red()
    btn.press_func(None)
    btn.release_func(None)
    tx1 = 0x83
    tx2 = 0x05
    tx3 = 0xE8

    while True:
        btn.press.clear()
        btn.release.clear()
        await btn.press.wait()
        logger.debug("Start game button pressed")
        await rgb.set_green()
        # IR TX send  
        logger.debug("Start game IR TX transmit")
        asyncio.create_task(ir_tx.transmit(tx1, tx2, tx3, False))
        #loop = asyncio.get_running_loop()
        #loop.set_exception_handler(exception_handler)
        await asyncio.sleep_ms(50)
        await btn.release.wait()
        logger.debug("Start game button released")
        await rgb.set_red()

async def olt_command(cmd, btn, rgb, ir_tx, tx1, tx2, tx3):
    logger.debug(f"Init OLT game command {cmd} on button {btn}")
    await rgb.set_red()
    btn.press_func(None)
    btn.release_func(None)
    
    while True:
        btn.press.clear()
        btn.release.clear()
        await btn.press.wait()
        logger.debug("Command button pressed")
        await rgb.set_green()
        # IR TX send  
        logger.debug("Command IR TX transmit")
        asyncio.create_task(ir_tx.transmit(tx1, tx2, tx3, False))
        #loop = asyncio.get_running_loop()
        #loop.set_exception_handler(exception_handler)
        await asyncio.sleep_ms(50)
        await btn.release.wait()
        logger.debug("Command button released")
        await rgb.set_red()



    
### main ###
async def main(proto):

    logger.debug("main start")
    
    ### alive check ###
    asyncio.create_task(alive_check())
    
    ### RGB ###
    logger.debug("RGB init")
    rgb1 = RGB()
    


    ### OLT IR RX ###
    logger.debug("OLT IR RX init")
    # User callback
    def cb(byte1, byte2, byte3, packet):
        print(f"byte1 0x{byte1:02x} byte2 0x{byte2:02x} byte3 0x{byte3:02x} packet 0x{packet:06x}")

    ir_rx = LT_24_RX(ir_rx_pin, cb)  # Instantiate receiver
    ir_rx.error_function(print_error)  # Show debug information 

    ### OLT IR TX ###
    logger.debug("OLT IR TX init")
    
    ir_tx = LT_24_TX(ir_tx_pin, 56000)  # My decoder chip is 56KHz
    # Uncomment the following to print transmit timing
    ir_tx.timeit = True


    ### Buttons ###
    logger.debug("Button init")
    pin1 = machine.Pin(20, machine.Pin.IN, machine.Pin.PULL_UP) # Button 1 on membrane switch
    btn1 = Pushbutton(pin1)
    pin2 = machine.Pin(21, machine.Pin.IN, machine.Pin.PULL_UP) # Button 2 on membrane switch
    btn2 = Pushbutton(pin2)
    pin3 = machine.Pin(18, machine.Pin.IN, machine.Pin.PULL_UP) # Button 3 on membrane switch
    btn3 = Pushbutton(pin3)
    pin4 = machine.Pin(19, machine.Pin.IN, machine.Pin.PULL_UP) # Button 4 on membrane switch
    btn4 = Pushbutton(pin4)

    COMMANDS[0].append(btn1)
    COMMANDS[1].append(btn2)
    #COMMANDS[2].append(btn3)
    #COMMANDS[3].append(btn4)
    
    #btn3.release_func(None)
    #btn3.double_func(None)
    #btn3.long_func(None)
    for command, tx1, tx2, btn in COMMANDS:
        logger.debug(f"command: {command} btn: {btn} tx1: {tx1} tx2: {tx2}")
        asyncio.create_task(olt_command(command, btn, rgb1, ir_tx, tx1, tx2, LAST_BYTE))

    #asyncio.create_task(start_game_comand(btn3, rgb1, ir_tx))

    ### Main loop ###
    while True:
        await asyncio.sleep(1)

def exception_handler(loop, context):
	# log exception
    print(context['exception'])

### asyncio start main ###
try:
    
    asyncio.run(main(0))
    

except KeyboardInterrupt:
    logger.exception("App Interrupted")

finally:
    asyncio.new_event_loop()
    logger.exception('End')

